︠98b96fc8-8316-4120-83d9-74e2fd4f27d7︠
import networkx.generators.atlas
atlas_graphs = [Graph(i) for i in networkx.generators.atlas.graph_atlas_g()]
︡713f1185-9ace-480a-88e8-f8ead8aabf1d︡︡
︠cc71704b-d410-4be0-8b86-8a95cdf2ead7︠
def minimum_rank(F, g):
    """
    Returns the minimum rank of the graph g over the field F.  This is done by exhaustively iterating 
    through all possible matrices in S(F,G).
    
    EXAMPLES:
        
        sage: min_rank, mat = minimum_rank(FiniteField(3), graphs.PathGraph(4))
        sage: min_rank
        3
        sage: mat
        [1 1 0 0]
        [1 1 1 0]
        [0 1 0 1]
        [0 0 1 0]
    """
    A=g.adjacency_matrix().change_ring(F)
    n=g.order()
    nonzero_elements = [e for e in F if e != 0]
    all_possible_nonzero_entries = CartesianProduct(*[nonzero_elements]*g.size())
    nonzero_positions = set(g.edges(labels=False))
    diagonals = VectorSpace(F, n) # The vector space of n-dimensional vectors over F

    minimum_rank = n

    for nonzero_entries in all_possible_nonzero_entries:
        #print "testing nonzero entries: ", nonzero_entries
        
        # Set the nonzero positions
        for position, entry in zip(nonzero_positions, nonzero_entries):
            A[position] = entry
            # and the symmetric entry
            A[position[1], position[0]] = entry
    
        # Iterate through all the diagonals
        for diagonal in diagonals:
            for index, diagonal_entry in enumerate(diagonal):
                A[index, index] = diagonal_entry

            new_rank = A.rank()
            if new_rank < minimum_rank:
                minimum_rank = new_rank
                minimum_rank_matrix = A.copy()
                print "min rank example: rank ", minimum_rank
                print minimum_rank_matrix
                print sage_input(minimum_rank_matrix)
                print

    return minimum_rank, minimum_rank_matrix
︡9740740a-74ff-4215-9c7e-7a949e01ed32︡︡
︠8d8e9aca-64db-440e-bafe-84eac61c3963︠
g=atlas_graphs[1124]
show(g)
︡38e505d4-8a6b-4e10-9684-d8733a321abb︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠50804ef6-cbeb-4c06-be03-103093d413f5i︠
%html
<p>The minimum rank function calculation below takes a long time because it is iterating through almost 36 million matrices.&nbsp; To see this, off-diagonal nonzero entries can be one of two values, and diagonal entries can be one of three values, so we have:</p>

︡67b55397-b221-4018-9450-59d444a9b212︡{"html": "<p>The minimum rank function calculation below takes a long time because it is iterating through almost 36 million matrices.&nbsp; To see this, off-diagonal nonzero entries can be one of two values, and diagonal entries can be one of three values, so we have:</p>"}︡
︠f90df226-d5f3-44c1-be4e-1704088a5365︠
2^g.size()*3^g.order()
︡158aac9a-0f5d-4cae-9b7f-62f988aff7b0︡{"stdout": "35831808"}︡
︠c4e1ba8b-4fa5-4d7c-8a47-e9339709c661i︠
%html
<p>However, we can interrupt the calculation after just a few seconds since we know that the minimum rank is at least 3 from zero forcing, and we quickly find a matrix with rank 3 over the finite field of size 3.</p>

︡89a86e70-84ba-48db-8d5e-ecc79c92ff37︡{"html": "<p>However, we can interrupt the calculation after just a few seconds since we know that the minimum rank is at least 3 from zero forcing, and we quickly find a matrix with rank 3 over the finite field of size 3.</p>"}︡
︠19c9be12-4ee6-45e8-afc7-6e443c41c384︠
minimum_rank(FiniteField(3), g)
︡bcf3a835-4439-4032-b822-41c5a451b554︡{"stdout": "min rank example: rank  6\n[0 1 1 1 1 0 0]\n[1 0 1 1 1 0 0]\n[1 1 0 1 1 0 0]\n[1 1 1 0 1 1 0]\n[1 1 1 1 0 1 1]\n[0 0 0 1 1 0 1]\n[0 0 0 0 1 1 0]\nmatrix(GF(3), [[0, 1, 1, 1, 1, 0, 0], [1, 0, 1, 1, 1, 0, 0], [1, 1, 0, 1, 1, 0, 0], [1, 1, 1, 0, 1, 1, 0], [1, 1, 1, 1, 0, 1, 1], [0, 0, 0, 1, 1, 0, 1], [0, 0, 0, 0, 1, 1, 0]])\n\nmin rank example: rank  5\n[1 1 1 1 1 0 0]\n[1 1 1 1 1 0 0]\n[1 1 1 1 1 0 0]\n[1 1 1 0 1 1 0]\n[1 1 1 1 0 1 1]\n[0 0 0 1 1 0 1]\n[0 0 0 0 1 1 0]\nmatrix(GF(3), [[1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 0, 1, 1, 0], [1, 1, 1, 1, 0, 1, 1], [0, 0, 0, 1, 1, 0, 1], [0, 0, 0, 0, 1, 1, 0]])\n\nmin rank example: rank  4\n[1 1 1 1 1 0 0]\n[1 1 1 1 1 0 0]\n[1 1 1 1 1 0 0]\n[1 1 1 2 1 1 0]\n[1 1 1 1 1 1 1]\n[0 0 0 1 1 0 1]\n[0 0 0 0 1 1 0]\nmatrix(GF(3), [[1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 2, 1, 1, 0], [1, 1, 1, 1, 1, 1, 1], [0, 0, 0, 1, 1, 0, 1], [0, 0, 0, 0, 1, 1, 0]])\n\nmin rank example: rank  3\n[1 1 1 1 1 0 0]\n[1 1 1 1 1 0 0]\n[1 1 1 1 1 0 0]\n[1 1 1 0 1 1 0]\n[1 1 1 1 2 1 1]\n[0 0 0 1 1 0 1]\n[0 0 0 0 1 1 1]\nmatrix(GF(3), [[1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 1, 1, 0, 0], [1, 1, 1, 0, 1, 1, 0], [1, 1, 1, 1, 2, 1, 1], [0, 0, 0, 1, 1, 0, 1], [0, 0, 0, 0, 1, 1, 1]])"}︡
︠10d3b513-c549-47c7-8849-469331554897︠
g.clique_number()
︡00d35586-3e34-42a8-a4c3-5438a6d7f35d︡{"stdout": "5"}︡
